#!/bin/bash
#
# Copyright (c) 2021, Oracle and/or its affiliates.
#

set -o nounset -o errexit -o pipefail
set -m

SPRITE_X11_DIR_DEFAULT=/opt/oracle/sprite-x11

SPRITE_X11_DIR=${SPRITE_X11_HOME:-${SPRITE_X11_DIR_DEFAULT}}
SPRITE_X11_BIN="${SPRITE_X11_DIR}/bin"

XAUTH_FILE="${HOME}/.Xauthority"

# Pull in common config
source "${SPRITE_X11_BIN}/common.sh"

# Pull in logging levels
source "${SPRITE_X11_BIN}/logging.sh"

# Pull in statuses
source "${SPRITE_X11_BIN}/status.sh"

SSHD="sshd"

display_usage()
{
    echo "Usage:"
    echo "  x11SessionService.sh [OPTION…] -- <command> [args]"
    echo ""
    echo "Options:"
    echo "  -h|--help               This help message."
    echo "  -W|--width <width>      Width in pixels."
    echo "  -H|--height <height>    Height in pixels."
    echo "  -f|--xauth-file <FILE>  Xauth file."
    echo "  -a|--audio-rate <rate>  Audio sample rate (samples/sec)."
    echo ""
    echo "Arguments:"
    echo "  <command>       Application command to run. Required."
    echo "  <args>          Arguments to application command. Optional."
}

exit_failure()
{
    local errorCode=${1}
    local errorString="${STATUSES[${errorCode}]}"
    echo "status:${errorString}"
    exit ${errorCode}
}

# Create a fifo that can be used by the igr server
# to write its params to
create_params_fifo()
{
    local TEMPLATE="/tmp/x11sessionservice.XXXXXXXXXX"
    local ACCESS_PERMS=600
    local dir_path=`mktemp -d ${TEMPLATE}`
    if [ $? -ne 0 ]
    then 
        exit_failure ${STATUS_E_MKTEMPFAILED}
    fi
    local fifo_path="${dir_path}/params_fifo"
    mkfifo -m${ACCESS_PERMS}  "${fifo_path}"
    if [ $? -ne 0 ]
    then 
        exit_failure ${STATUS_E_MKFIFOFAILED}
    fi
    echo "${fifo_path}"
}

# Remove the params fifo 
# @param 1 fifo path
destroy_params_fifo()
{
    local fifo_path="${1}"
    rm "${fifo_path}"
    rmdir `dirname "${fifo_path}"`
}

# Get the authtoken from the given fifo
# @param 1 fifo path to get the token from
get_auth_token()
{
    local fifo_path="${1}"
    local authtoken=`head -n1 "${fifo_path}" | sed 's/authtoken://'`
    echo "${authtoken}"
}

echo "x11SessionService" >> ${LOGFILE} 2>&1
date >> ${LOGFILE} 2>&1

set +e
options=$(getopt -q -o hW:H:f:a: -l help,width:,height:,xauth-file:,audio-rate: -- "$@")
if [ $? -ne 0 ]
then
    exit_failure ${STATUS_E_UNKNOWNARG}
fi
set -e

AUDIO_PORT=""
AUDIO_ARGS=""

echo "Run x11AppService" >> ${LOGFILE} 2>&1
APP_SERVICE_VARS=`setsid "${SPRITE_X11_BIN}/x11AppService.sh" $@`
read X_PORT AUDIO_PORT <<< ${APP_SERVICE_VARS}
echo "Got X port ${X_PORT}, audio port ${AUDIO_PORT}" >> ${LOGFILE} 2>&1

if [ -n "${AUDIO_PORT}" ]
then
    AUDIO_ARGS="--enable-audio ${AUDIO_PORT}"
fi

# Set the environment
DISPLAY=":${X_PORT}"
export DISPLAY
XAUTHORITY=${XAUTH_FILE}
export XAUTHORITY

set +e
if [ "${SPRITE_X11_SINGLE_INSTANCE}" == "1" ]
then
    # 9876 is the only sprite port open in the firewall, so it is the only
    # one we can use
    SPRITE_PORT=9876

    # If igrserver happens to be running from a previous instance,
    # try to kill it
    killall igrserver-xdamage
else
    SPRITE_PORT="$(${SPRITE_X11_BIN}/findport.py igrserver-xdamage 9876 9776)"
    if [ $? -ne 0 ]
    then
        exit_failure ${STATUS_E_NOFREESPRITEPORT}
    fi
fi
set -e

PARAMS_FIFO=`create_params_fifo`

# Run the igrserver
"${SPRITE_X11_BIN}/igrserver-xdamage" \
        -L ${SPRITE_X11_IGRSERVER_LOG_LEVEL} \
        --enable-reduce \
        --disable-rca \
        --enable-ssl \
        --enable-clipboard \
        --set-keyboard-model 1 \
        --enable-tokenauth \
        ${AUDIO_ARGS} \
        --output-params-file "${PARAMS_FIFO}" \
        -p "${SPRITE_PORT}" &> /dev/null &
IGRSERVER_PID=$!
echo "igrserver-xdamage pid ${IGRSERVER_PID}" >> ${LOGFILE} 2>&1

AUTHTOKEN=`get_auth_token ${PARAMS_FIFO}`
destroy_params_fifo "${PARAMS_FIFO}"
echo "status:${STATUSES[${STATUS_S_OK}]} port:${SPRITE_PORT} authtoken:${AUTHTOKEN}"

terminate()
{
    echo "x11SessionService terminate" >> ${LOGFILE} 2>&1
    date >> ${LOGFILE} 2>&1

    trap '' CHLD
    set +e
    remove_xauth_cookie
    # Kill any process that might be left
    kill "${IGRSERVER_PID}" &>/dev/null
    # Remove any lock files
    ${SPRITE_X11_BIN}/findport.py igrserver-xdamage -r ${SPRITE_PORT}
    # Kill parent sshd process, needed to end CDM.
    PARENT_PROCESS="$(ps -q "$PPID" -o comm=)"
    if [ "${PARENT_PROCESS}" == "${SSHD}" ]
    then
        kill "${PPID}" &>/dev/null
    fi
    set -e
}

trap terminate CHLD TERM SIGINT SIGHUP

wait

exit ${STATUS_S_OK}
